'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeZoomFactor = exports.getMaxZoomIntensity = exports.isTwoFingers = exports.pad = exports.removeClass = exports.addClass = exports.contain = exports.getSizeAccordingToPixelRatio = exports.getResponsiveWidthOfContainer = exports.magnify = exports.setView360Icon = exports.get360ViewProps = undefined;

var _ci = require('./ci360.constants');

//TODO [deprecated]: remove filename, amount in the upcoming versions
var get360ViewProps = function get360ViewProps(image) {
  return {
    folder: attr(image, 'folder') || attr(image, 'data-folder') || '/',
    filenameX: attr(image, 'filename') || attr(image, 'data-filename') || attr(image, 'filename-x') || attr(image, 'data-filename-x') || 'image-{index}.jpg',
    filenameY: attr(image, 'filename-y') || attr(image, 'data-filename-y') || 'image-y-{index}.jpg',
    imageList: attr(image, 'image-list') || attr(image, 'data-image-list') || null,
    indexZeroBase: parseInt(attr(image, 'index-zero-base') || attr(image, 'data-index-zero-base') || 0, 10),
    amountX: parseInt(attr(image, 'amount') || attr(image, 'data-amount') || attr(image, 'amount-x') || attr(image, 'data-amount-x') || 36, 10),
    amountY: parseInt(attr(image, 'amount-y') || attr(image, 'data-amount-y') || 0, 10),
    imageOffset: parseInt(attr(image, 'image-offset') || attr(image, 'data-image-offset')),
    speed: parseInt(attr(image, 'speed') || attr(image, 'data-speed') || 80, 10),
    dragSpeed: parseInt(attr(image, 'drag-speed') || attr(image, 'data-drag-speed') || 150, 10),
    keys: isTrue(image, 'keys'),
    boxShadow: attr(image, 'box-shadow') || attr(image, 'data-box-shadow'),
    autoplay: isTrue(image, 'autoplay'),
    autoplayBehavior: attr(image, 'autoplay-behavior') || attr(image, 'data-autoplay-behavior') || _ci.AUTOPLAY_BEHAVIOR.SPIN_X,
    playOnce: isTrue(image, 'play-once'),
    disablePointerZoom: isTrue(image, 'disable-pointer-zoom'),
    disablePinchZoom: isTrue(image, 'disable-pinch-zoom'),
    onMouseLeave: attr(image, 'on-mouse-leave') || attr(image, 'data-on-mouse-leave'),
    toStartPointerZoom: attr(image, 'to-start-pointer-zoom') || attr(image, 'data-to-start-pointer-zoom') || _ci.TO_START_POINTER_ZOOM.SCROLL_TO_START,
    pointerZoomFactor: parseInt(attr(image, 'pointer-zoom-factor') || attr(image, 'data-pointer-zoom-factor') || 2, 10),
    pinchZoomFactor: parseInt(attr(image, 'pinch-zoom-factor') || attr(image, 'data-pinch-zoom-factor') || 2, 10),
    maxScale: parseFloat(attr(image, 'max-scale') || attr(image, 'data-max-scale') || 100, 10),
    autoplayReverse: isTrue(image, 'autoplay-reverse'),
    bottomCircle: isTrue(image, 'bottom-circle'),
    disableDrag: isTrue(image, 'disable-drag'),
    fullscreen: isTrue(image, 'fullscreen'),
    magnifier: (attr(image, 'magnifier') !== null || attr(image, 'data-magnifier') !== null) && parseInt(attr(image, 'magnifier') || attr(image, 'data-magnifier'), 10),
    magnifyInFullscreen: isTrue(image, 'magnify-in-fullscreen'),
    bottomCircleOffset: parseInt(attr(image, 'bottom-circle-offset') || attr(image, 'data-bottom-circle-offset') || 5, 10),
    ratio: parseFloat(attr(image, 'ratio') || attr(image, 'data-ratio') || 0) || false,
    responsive: isTrue(image, 'responsive'),
    ciToken: attr(image, 'responsive') || attr(image, 'data-responsive') || 'demo',
    ciFilters: attr(image, 'filters') || attr(image, 'data-filters'),
    ciTransformation: attr(image, 'transformation') || attr(image, 'data-transformation'),
    lazyload: isTrue(image, 'lazyload'),
    lazySelector: attr(image, 'lazyload-selector') || attr(image, 'data-lazyload-selector') || 'lazyload',
    spinReverse: isTrue(image, 'spin-reverse'),
    controlReverse: isTrue(image, 'control-reverse'),
    stopAtEdges: isTrue(image, 'stop-at-edges'),
    hide360Logo: isTrue(image, 'hide-360-logo'),
    logoSrc: attr(image, 'logo-src') || 'https://scaleflex.ultrafast.io/https://scaleflex.airstore.io/filerobot/js-cloudimage-360-view/360_view.svg'
  };
};

var isTrue = function isTrue(image, type) {
  var imgProp = attr(image, type);
  var imgDataProp = attr(image, 'data-' + type);

  return imgProp !== null && imgProp !== 'false' || imgDataProp !== null && imgDataProp !== 'false';
};

var attr = function attr(element, attribute) {
  return element.getAttribute(attribute);
};

var setView360Icon = function setView360Icon(view360Icon, logoSrc) {
  view360Icon.style.background = 'rgba(255,255,255,0.8) url(\'' + logoSrc + '\') 50% 50% / contain no-repeat';
};

var magnify = function magnify(container) {
  var offset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var src = arguments[2];
  var glass = arguments[3];
  var zoom = arguments[4];

  var w = void 0,
      h = void 0,
      bw = void 0;
  var _offset$x = offset.x,
      offsetX = _offset$x === undefined ? 0 : _offset$x,
      _offset$y = offset.y,
      offsetY = _offset$y === undefined ? 0 : _offset$y;

  var backgroundSizeX = (container.offsetWidth - offsetX * 2) * zoom;
  var backgroundSizeY = (container.offsetHeight - offsetY * 2) * zoom;

  glass.setAttribute("class", "cloudimage-360-img-magnifier-glass");
  container.prepend(glass);

  glass.style.backgroundImage = "url('" + src + "')";
  glass.style.backgroundSize = backgroundSizeX + 'px ' + backgroundSizeY + 'px';

  bw = 3;
  w = glass.offsetWidth / 2;
  h = glass.offsetHeight / 2;

  glass.addEventListener("mousemove", moveMagnifier);
  container.addEventListener("mousemove", moveMagnifier);

  glass.addEventListener("touchmove", moveMagnifier, { passive: true });
  container.addEventListener("touchmove", moveMagnifier, { passive: true });

  function moveMagnifier(e) {
    var pos = void 0,
        x = void 0,
        y = void 0;

    pos = getCursorPos(e);
    x = pos.x;
    y = pos.y;

    if (x > container.offsetWidth - w / zoom) {
      x = container.offsetWidth - w / zoom;
    }

    if (x < w / zoom) {
      x = w / zoom;
    }

    if (y > container.offsetHeight - h / zoom) {
      y = container.offsetHeight - h / zoom;
    }

    if (y < h / zoom) {
      y = h / zoom;
    }

    glass.style.left = x - w + "px";
    glass.style.top = y - h + "px";

    var backgroundPosX = (x - offsetX) * zoom - w + bw;

    var backgroundPosY = (y - offsetY) * zoom - h + bw;

    glass.style.backgroundPosition = '-' + backgroundPosX + 'px -' + backgroundPosY + 'px';
  }

  function getCursorPos(e) {
    var a = void 0,
        x = 0,
        y = 0;
    e = e || window.event;
    a = container.getBoundingClientRect();
    x = e.pageX - a.left;
    y = e.pageY - a.top;
    x = x - window.pageXOffset;
    y = y - window.pageYOffset;

    return { x: x, y: y };
  }
};

var getSizeLimit = function getSizeLimit(currentSize) {
  if (currentSize <= 25) return '25';
  if (currentSize <= 50) return '50';

  return (Math.ceil(currentSize / 100) * 100).toString();
};

var getSizeAccordingToPixelRatio = function getSizeAccordingToPixelRatio(size) {
  var splittedSizes = size.toString().split('x');
  var result = [];

  [].forEach.call(splittedSizes, function (size) {
    result.push(size * Math.round(window.devicePixelRatio || 1));
  });

  return result.join('x');
};

var getResponsiveWidthOfContainer = function getResponsiveWidthOfContainer(width) {
  return getSizeLimit(width);
};

var fit = function fit(contains) {
  return function (parentWidth, parentHeight, childWidth, childHeight) {
    var scale = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1;
    var offsetX = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0.5;
    var offsetY = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : 0.5;

    var childRatio = childWidth / childHeight;
    var parentRatio = parentWidth / parentHeight;
    var width = parentWidth * scale;
    var height = parentHeight * scale;

    if (contains ? childRatio > parentRatio : childRatio < parentRatio) {
      height = width / childRatio;
    } else {
      width = height * childRatio;
    }

    return {
      width: width,
      height: height,
      offsetX: (parentWidth - width) * offsetX,
      offsetY: (parentHeight - height) * offsetY
    };
  };
};

var isTwoFingers = function isTwoFingers(event) {
  return event.targetTouches.length === 2;
};

var getMaxZoomIntensity = function getMaxZoomIntensity(width, maxScale) {
  var maxWidth = maxScale * width;
  var maxIntensity = maxWidth - width;

  return maxIntensity;
};

var normalizeZoomFactor = function normalizeZoomFactor(event, pointerZoomFactor) {
  var scrollEvent = Math.abs(event.deltaY);
  var zoomFactor = scrollEvent < 125 ? -pointerZoomFactor * 10 : -pointerZoomFactor;

  return zoomFactor;
};

var contain = fit(true);

var addClass = function addClass(el, className) {
  if (el.classList) el.classList.add(className);else el.className += ' ' + className;
};

var removeClass = function removeClass(el, className) {
  if (el.classList) el.classList.remove(className);else el.className = el.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
};

var pad = function pad(n) {
  var width = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  n = n + '';

  return n.length >= width ? n : new Array(width - n.length + 1).join('0') + n;
};

exports.get360ViewProps = get360ViewProps;
exports.setView360Icon = setView360Icon;
exports.magnify = magnify;
exports.getResponsiveWidthOfContainer = getResponsiveWidthOfContainer;
exports.getSizeAccordingToPixelRatio = getSizeAccordingToPixelRatio;
exports.contain = contain;
exports.addClass = addClass;
exports.removeClass = removeClass;
exports.pad = pad;
exports.isTwoFingers = isTwoFingers;
exports.getMaxZoomIntensity = getMaxZoomIntensity;
exports.normalizeZoomFactor = normalizeZoomFactor;